@@ -20,6 +20,10 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+
 #include <glib/gstdio.h>
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
@@ -58,6 +62,7 @@ struct _MyPlugin
   GtkWidget            *image;
   GtkWidget            *menu;
   GtkWidget            *popup_menu;
+  gulong                popup_menu_id;
 };
 
 /*
@@ -75,6 +80,7 @@ static gboolean         cb_status_icon_set_size         (MyPlugin *plugin,
                                                          gint size);
 static void             cb_status_icon_finalize         (MyPlugin *plugin);
 static void             install_autostart_file          ();
+static void             update_autostart_file           ();
 
 /*
  * Panel Plugin
@@ -107,6 +113,8 @@ static void             cb_about_dialog_url_hook        (GtkAboutDialog *dialog,
                                                          const gchar *uri,
                                                          gpointer user_data);
 static void             plugin_configure                (MyPlugin *plugin);
+static void             plugin_block_menu               (MyPlugin *plugin);
+static void             plugin_unblock_menu             (MyPlugin *plugin);
 static void             cb_button_toggled               (GtkToggleButton *button,
                                                          MyPlugin *plugin);
 static void             cb_menu_deactivate              (GtkMenuShell *menu,
@@ -247,6 +255,8 @@ plugin_register ()
   plugin->collector = clipman_collector_get ();
   xfconf_g_property_bind (plugin->channel, "/settings/add-primary-clipboard",
                           G_TYPE_BOOLEAN, plugin->collector, "add-primary-clipboard");
+  xfconf_g_property_bind (plugin->channel, "/settings/history-ignore-primary-clipboard",
+                          G_TYPE_BOOLEAN, plugin->collector, "history-ignore-primary-clipboard");
   xfconf_g_property_bind (plugin->channel, "/settings/enable-actions",
                           G_TYPE_BOOLEAN, plugin->collector, "enable-actions");
   xfconf_g_property_bind (plugin->channel, "/inhibit",
@@ -285,8 +295,9 @@ status_icon_register ()
   /* Signals */
   g_signal_connect_swapped (plugin->status_icon, "activate",
                             G_CALLBACK (cb_status_icon_activate), plugin);
-  g_signal_connect_swapped (plugin->status_icon, "popup-menu",
-                            G_CALLBACK (cb_status_icon_popup_menu), plugin);
+  plugin->popup_menu_id =
+    g_signal_connect_swapped (plugin->status_icon, "popup-menu",
+                              G_CALLBACK (cb_status_icon_popup_menu), plugin);
   g_signal_connect_swapped (plugin->status_icon, "size-changed",
                             G_CALLBACK (cb_status_icon_set_size), plugin);
   g_object_weak_ref (G_OBJECT (plugin->status_icon), (GWeakNotify)cb_status_icon_finalize, plugin);
@@ -319,9 +330,6 @@ cb_status_icon_popup_menu (MyPlugin *plugin, guint button, guint activate_time)
 {
   GtkWidget *mi;
 
-  if (plugin->gxml != NULL)
-    return;
-
   if (G_UNLIKELY (plugin->popup_menu == NULL))
     {
       plugin->popup_menu = gtk_menu_new ();
@@ -353,6 +361,12 @@ cb_status_icon_popup_menu (MyPlugin *plugin, guint button, guint activate_time)
 static void
 cb_status_icon_quit (MyPlugin *plugin)
 {
+  gint res;
+  res = xfce_message_dialog (NULL, _("Autostart Clipman"), GTK_STOCK_DIALOG_QUESTION,
+                             _("Autostart Clipman"), _("Do you want to restart the clipboard manager the next time you login?"),
+                             GTK_STOCK_YES, GTK_RESPONSE_YES, GTK_STOCK_NO, GTK_RESPONSE_NO, NULL);
+  update_autostart_file (res == GTK_RESPONSE_YES);
+
   gtk_status_icon_set_visible (plugin->status_icon, FALSE);
   gtk_main_quit ();
 }
@@ -362,9 +376,7 @@ cb_status_icon_set_size (MyPlugin *plugin, gint size)
 {
   GdkPixbuf *pixbuf;
  
-  pixbuf = xfce_themed_icon_load ("xfce4-clipman-plugin", size);
-  if (pixbuf == NULL)
-    pixbuf = xfce_themed_icon_load (GTK_STOCK_PASTE, size);
+  pixbuf = xfce_themed_icon_load (GTK_STOCK_PASTE, size);
   gtk_status_icon_set_from_pixbuf (plugin->status_icon, pixbuf);
   g_object_unref (G_OBJECT (pixbuf));
 
@@ -424,6 +436,35 @@ out:
   g_free (userfile);
 }
 
+static void
+update_autostart_file (gboolean autostart)
+{
+  gchar *userfile;
+  gint res;
+  GKeyFile *keyfile;
+  gchar *data;
+
+  userfile = g_strdup_printf ("%s/autostart/"PACKAGE"-autostart.desktop", g_get_user_config_dir ());
+
+  /* Check if the user autostart file exists */
+  if (!g_file_test (userfile, G_FILE_TEST_EXISTS))
+    {
+      g_warning ("The autostart file (%s) is not installed, what did you do?", userfile);
+      g_free (userfile);
+      return;
+    }
+
+  /* Update the file */
+  keyfile = g_key_file_new ();
+  g_key_file_load_from_file (keyfile, userfile, G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
+  g_key_file_set_boolean (keyfile, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_HIDDEN, !autostart);
+  data = g_key_file_to_data (keyfile, NULL, NULL);
+  g_file_set_contents (userfile, data, -1, NULL);
+  g_free (data);
+  g_key_file_free (keyfile);
+  g_free (userfile);
+}
+
 /*
  * Panel Plugin
  */
@@ -478,9 +519,7 @@ panel_plugin_set_size (MyPlugin *plugin,
 
   size -= 2 + 2 * MAX (plugin->button->style->xthickness,
                        plugin->button->style->ythickness);
-  pixbuf = xfce_themed_icon_load ("xfce4-clipman-plugin", size);
-  if (pixbuf == NULL)
-    pixbuf = xfce_themed_icon_load (GTK_STOCK_PASTE, size);
+  pixbuf = xfce_themed_icon_load (GTK_STOCK_PASTE, size);
   gtk_image_set_from_pixbuf (GTK_IMAGE (plugin->image), pixbuf);
   g_object_unref (G_OBJECT (pixbuf));
 
@@ -639,13 +678,18 @@ plugin_free (MyPlugin *plugin)
 static void
 plugin_about (MyPlugin *plugin)
 {
-  static const gchar *artists[] = { "Mike Massonnet", NULL, };
-  static const gchar *authors[] = { "Mike Massonnet",
-                                    "", "Authors of previous versions:",
+  const gchar *artists[] = { "Mike Massonnet", NULL, };
+  const gchar *authors[] = { "Mike Massonnet",
+                                    "", _("Authors of previous versions:"),
                                     "Eduard Roccatello",
                                     "Nick Schermer",
                                     NULL, };
-  static const gchar *documenters[] = { "Mike Massonnet", NULL, };
+  const gchar *documenters[] = { "Mike Massonnet", NULL, };
+  const gchar *license =
+    "This program is free software; you can redistribute it and/or modify\n"
+    "it under the terms of the GNU General Public License as published by\n"
+    "the Free Software Foundation; either version 2 of the License, or\n"
+    "(at your option) any later version.\n";
 
   gtk_about_dialog_set_url_hook (cb_about_dialog_url_hook, NULL, NULL);
   gtk_show_about_dialog (NULL,
@@ -654,7 +698,7 @@ plugin_about (MyPlugin *plugin)
                          "comments", _("Clipboard Manager for Xfce"),
                          "version", PACKAGE_VERSION,
                          "copyright", "Copyright © 2008-2009 Mike Massonnet",
-                         "license", XFCE_LICENSE_GPL,
+                         "license", license,
                          "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-clipman-plugin",
                          "website-label", "goodies.xfce.org",
                          "artists", artists,
@@ -703,18 +747,24 @@ plugin_configure (MyPlugin *plugin)
   gtk_window_group_add_window (group, GTK_WINDOW (action_dialog));
 
   /* General settings */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (plugin->gxml, "save-on-quit")),
-                                DEFAULT_SAVE_ON_QUIT);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (plugin->gxml, "add-selections")),
                                 DEFAULT_ADD_PRIMARY_CLIPBOARD);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (plugin->gxml, "history-ignore-selections")),
+                                DEFAULT_HISTORY_IGNORE_PRIMARY_CLIPBOARD);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (plugin->gxml, "save-on-quit")),
+                                DEFAULT_SAVE_ON_QUIT);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (plugin->gxml, "store-an-image")),
                                 (gboolean)DEFAULT_MAX_IMAGES_IN_HISTORY);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (glade_xml_get_widget (plugin->gxml, "max-texts-in-history")),
                              (gdouble)DEFAULT_MAX_TEXTS_IN_HISTORY);
-  xfconf_g_property_bind (plugin->channel, "/settings/save-on-quit", G_TYPE_BOOLEAN,
-                          G_OBJECT (glade_xml_get_widget (plugin->gxml, "save-on-quit")), "active");
   xfconf_g_property_bind (plugin->channel, "/settings/add-primary-clipboard", G_TYPE_BOOLEAN,
                           G_OBJECT (glade_xml_get_widget (plugin->gxml, "add-selections")), "active");
+  xfconf_g_property_bind (plugin->channel, "/settings/add-primary-clipboard", G_TYPE_BOOLEAN,
+                          G_OBJECT (glade_xml_get_widget (plugin->gxml, "history-ignore-selections")), "sensitive");
+  xfconf_g_property_bind (plugin->channel, "/settings/history-ignore-primary-clipboard", G_TYPE_BOOLEAN,
+                          G_OBJECT (glade_xml_get_widget (plugin->gxml, "history-ignore-selections")), "active");
+  xfconf_g_property_bind (plugin->channel, "/settings/save-on-quit", G_TYPE_BOOLEAN,
+                          G_OBJECT (glade_xml_get_widget (plugin->gxml, "save-on-quit")), "active");
   xfconf_g_property_bind (plugin->channel, "/settings/max-images-in-history", G_TYPE_UINT,
                           G_OBJECT (glade_xml_get_widget (plugin->gxml, "store-an-image")), "active");
   xfconf_g_property_bind (plugin->channel, "/settings/max-texts-in-history", G_TYPE_UINT,
@@ -738,7 +788,7 @@ plugin_configure (MyPlugin *plugin)
   setup_actions_treeview (GTK_TREE_VIEW (glade_xml_get_widget (plugin->gxml, "actions")), plugin);
   setup_commands_treeview (GTK_TREE_VIEW (glade_xml_get_widget (plugin->gxml, "commands")), plugin);
 
-  /* Callbacks for the OK button sensitivity in the actions dialog */
+  /* Callbacks for the OK button sensitivity in the edit action dialog */
   g_signal_connect_after (glade_xml_get_widget (plugin->gxml, "action-name"), "changed",
                           G_CALLBACK (cb_set_action_dialog_button_ok), plugin);
   g_signal_connect_after (glade_xml_get_widget (plugin->gxml, "regex"), "changed",
@@ -749,11 +799,9 @@ plugin_configure (MyPlugin *plugin)
                           G_CALLBACK (cb_set_action_dialog_button_ok), plugin);
 
   /* Run the dialog */
-  if (plugin->panel_plugin != NULL)
-    xfce_panel_plugin_block_menu (plugin->panel_plugin);
+  plugin_block_menu (plugin);
   while (gtk_dialog_run (GTK_DIALOG (dialog)) != 0);
-  if (plugin->panel_plugin != NULL)
-    xfce_panel_plugin_unblock_menu (plugin->panel_plugin);
+  plugin_unblock_menu (plugin);
 
   gtk_widget_destroy (action_dialog);
   gtk_widget_destroy (dialog);
@@ -765,6 +813,32 @@ plugin_configure (MyPlugin *plugin)
   clipman_actions_save (plugin->actions);
 }
 
+static void
+plugin_block_menu (MyPlugin *plugin)
+{
+  if (plugin->panel_plugin != NULL)
+    {
+      xfce_panel_plugin_block_menu (plugin->panel_plugin);
+    }
+  else
+    {
+      g_signal_handler_block (plugin->status_icon, plugin->popup_menu_id);
+    }
+}
+
+static void
+plugin_unblock_menu (MyPlugin *plugin)
+{
+  if (plugin->panel_plugin != NULL)
+    {
+      xfce_panel_plugin_unblock_menu (plugin->panel_plugin);
+    }
+  else
+    {
+      g_signal_handler_unblock (plugin->status_icon, plugin->popup_menu_id);
+    }
+}
+
 static void
 cb_button_toggled (GtkToggleButton *button,
                    MyPlugin *plugin)
@@ -854,8 +928,7 @@ my_plugin_take_ownership ()
   if (XGetSelectionOwner (display, atom))
     {
       dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                       _("There is already a clipboard manager running"),
-                                       NULL);
+                                       _("There is already a clipboard manager running"));
       gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
@@ -938,23 +1011,67 @@ cb_show_help (GtkButton *button,
               MyPlugin *plugin)
 {
   GdkScreen *screen;
+  gchar *locale = NULL;
+  gchar *offset;
   gchar *filename = NULL;
   gchar *command = NULL;
   
+#ifdef ENABLE_NLS
+#ifdef HAVE_LOCALE_H
+  locale = g_strdup (setlocale (LC_ALL, ""));
+  if (locale != NULL)
+    {
+      offset = g_strrstr (locale, ".");
+      if (offset != NULL)
+        *offset = '\0';
+    }
+  else
+    locale = g_strdup ("C");
+#else
+  locale = g_strdup ("C");
+#endif
+
+  filename = g_strdup_printf (DATAROOTDIR"/xfce4/doc/%s/"PACKAGE".html", locale);
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+    {
+      offset = g_strrstr (locale, "_");
+      if (offset == NULL)
+        {
+          g_free (filename);
+          filename = g_strdup (DATAROOTDIR"/xfce4/doc/C/"PACKAGE".html");
+        }
+      else
+        {
+          *offset = '\0';
+          g_free (filename);
+          filename = g_strdup_printf (DATAROOTDIR"/xfce4/doc/%s/"PACKAGE".html", locale);
+          if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+            {
+              g_free (filename);
+              filename = g_strdup (DATAROOTDIR"/xfce4/doc/C/"PACKAGE".html");
+            }
+        }
+    }
+
+  g_free (locale);
+#else
+  filename = g_strdup (DATAROOTDIR"/xfce4/doc/C/"PACKAGE".html");
+#endif
+
   screen = gtk_widget_get_screen (GTK_WIDGET (button));
-  filename = "file://"DATAROOTDIR"/xfce4/doc/C/"PACKAGE".html";
-  command = g_strdup_printf ("exo-open %s", filename);
+  command = g_strdup_printf ("exo-open file://%s", filename);
   if (gdk_spawn_command_line_on_screen (screen, command, NULL))
     goto out;
 
   g_free (command);
-  command = g_strdup_printf ("firefox %s", filename);
+  command = g_strdup_printf ("firefox file://%s", filename);
   if (gdk_spawn_command_line_on_screen (screen, command, NULL))
     goto out;
 
   xfce_err ("Unable to open documentation \"%s\"", filename);
 
 out:
+  g_free (filename);
   g_free (command);
 }
 
@@ -1001,7 +1118,8 @@ refresh_actions_treeview (GtkTreeView *treeview,
     {
       entry = entries->data;
 
-      title = g_strdup_printf ("<b>%s</b>\n<small>%s</small>", entry->action_name, g_regex_get_pattern (entry->regex));
+      title = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>",
+                                       entry->action_name, g_regex_get_pattern (entry->regex));
       gtk_list_store_append (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, entry, 1, title, -1);
       g_free (title);
@@ -1130,7 +1248,7 @@ cb_actions_row_activated (GtkTreeView *treeview,
   g_hash_table_iter_init (&hiter, entry->commands);
   while (g_hash_table_iter_next (&hiter, &key, &value))
     {
-      title = g_strdup_printf ("<b>%s</b>\n<small>%s</small>", (gchar *)key, (gchar *)value);
+      title = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>", (gchar *)key, (gchar *)value);
       gtk_list_store_append (GTK_LIST_STORE (commands_model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (commands_model), &iter, 0, title, 1, key, 2, value, -1);
       g_free (title);
@@ -1221,7 +1339,7 @@ __foreach_command_fill_commands (gpointer key,
   GtkTreeIter iter;
   gchar *title;
 
-  title = g_strdup_printf ("<b>%s</b>\n<small>%s</small>", (gchar *)key, (gchar *)value);
+  title = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>", (gchar *)key, (gchar *)value);
   gtk_list_store_append (GTK_LIST_STORE (_model), &iter);
   gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, title, 1, key, 2, value, -1);
   g_free (title);
@@ -1275,10 +1393,10 @@ cb_add_command (GtkButton *button,
       || gtk_entry_get_text (GTK_ENTRY (command))[0] == '\0')
     return;
 
+  title = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>",
+                                   gtk_entry_get_text (GTK_ENTRY (command_name)),
+                                   gtk_entry_get_text (GTK_ENTRY (command)));
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (glade_xml_get_widget (plugin->gxml, "commands")));
-  title = g_strdup_printf ("<b>%s</b>\n<small>%s</small>",
-                           gtk_entry_get_text (GTK_ENTRY (command_name)),
-                           gtk_entry_get_text (GTK_ENTRY (command)));
   gtk_list_store_append (GTK_LIST_STORE (model), &iter);
   gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, title,
                       1, gtk_entry_get_text (GTK_ENTRY (command_name)),
@@ -1316,9 +1434,9 @@ cb_refresh_command (GtkButton *button,
       return;
     }
 
-  title = g_strdup_printf ("<b>%s</b>\n<small>%s</small>",
-                           gtk_entry_get_text (GTK_ENTRY (command_name)),
-                           gtk_entry_get_text (GTK_ENTRY (command)));
+  title = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>",
+                                   gtk_entry_get_text (GTK_ENTRY (command_name)),
+                                   gtk_entry_get_text (GTK_ENTRY (command)));
   gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, title,
                       1, gtk_entry_get_text (GTK_ENTRY (command_name)),
                       2, gtk_entry_get_text (GTK_ENTRY (command)), -1);
@@ -1367,7 +1485,8 @@ cb_set_action_dialog_button_ok (GtkWidget *widget,
 
   if (action_name[0] != '\0' && regex_pattern[0] != '\0' && has_commands)
     {
-      if (regex = g_regex_new (regex_pattern, 0, 0, NULL))
+      regex = g_regex_new (regex_pattern, 0, 0, NULL);
+      if (regex != NULL)
         {
           sensitive = TRUE;
           g_regex_unref (regex);
